/*
 * main implementation: use this 'C' sample to create your own application
 *
 */

#include "derivative.h" /* include peripheral declarations */
#include "MPC5744P.h"
#include "freemaster.h"

#define MAXCNT 100000

extern void xcptn_xmpl(void);
void Mcu_Sysclk_Init(void);
void Mcu_SCI_Init(void);
void init_all(void);

volatile int counter = 0;

void Mcu_Sysclk_Init(void)
{
    /* Set RUN Configuration Registers */
    MC_ME.RUN_PC[0].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[1].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[2].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[3].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[4].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[5].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[6].R=0x000000FE;               /* Peripheral ON in every mode  */
    MC_ME.RUN_PC[7].R=0x000000FE;               /* Peripheral ON in every mode  */

    MC_CGM.SC_DC0.B.DIV = 1;                    /* System clock divider is 2 */

    MC_CGM.AC2_DC0.B.DE = 1;                    /* Enable CAN clock */
    MC_CGM.AC2_DC0.B.DIV = 1;                   /* CAN clock divider is 2 */
    MC_CGM.AC3_SC.B.SELCTL = 1;                 /* PLL0 reference clock is XOSC */

    /* Fpll = 40 MHz * 64 / (5 * 8) = 64 MHz */
    PLLDIG.PLL0DV.B.PREDIV = 5;
    PLLDIG.PLL0DV.B.MFD = 64;
    PLLDIG.PLL0DV.B.RFDPHI = 8;

    /* Configure system behavior during DRUN mode */
    MC_ME.DRUN_MC.B.XOSCON = 1;                 /* External cristal oscilator is switch on */
    MC_ME.DRUN_MC.B.PLL0ON = 1;                 /* Primary pll is switched on */
    MC_ME.DRUN_MC.B.SYSCLK = 2;                 /* PLL0 PHI - sysclk now */

    MC_ME.MCTL.R = 0x30005AF0;                  /* Write Mode and Key */
    MC_ME.MCTL.R = 0x3000A50F;                  /* Write Mode and Key inverted */

    while(MC_ME.GS.B.S_MTRANS == 1);            /* Wait for mode entry complete  */
    while(MC_ME.GS.B.S_CURRENT_MODE != 0x3);    /* Check DRUN mode entered */
}

void Mcu_Linflex_0_Init()
{
    /* Configure linFlexD2 */
    LINFlexD_0.LINCR1.B.SLEEP = 0x0;      /* Disable sleep mode */
    LINFlexD_0.LINCR1.B.INIT  = 0x1;      /* Init mode */
    LINFlexD_0.UARTCR.B.UART  = 0x1;      /* UART mode */

    LINFlexD_0.UARTCR.B.TDFL_TFC = 0x0;   /* Buffer size = 1 */
    LINFlexD_0.UARTCR.B.RDFL_RFC = 0x0;   /* Buffer size = 1 */
    LINFlexD_0.UARTCR.B.RxEn = 0x1;       /* Receiver enable */
    LINFlexD_0.UARTCR.B.TxEn = 0x1;       /* Transmitter enable */
    LINFlexD_0.UARTCR.B.PC0  = 0x1;       /* Parity is even */
    LINFlexD_0.UARTCR.B.PCE  = 0x0;       /* Parity control disable */
    LINFlexD_0.UARTCR.B.WL0  = 0x1;       /* Word length 8-bit data + parity bit */
    LINFlexD_0.LINFBRR.B.FBR = 5;
    LINFlexD_0.LINIBRR.B.IBR = 17;        /* Baud rate = 115200, in case Fpbridge0 = 32 MHz */

    LINFlexD_0.LINCR1.B.INIT = 0x0;       /* Normal mode */

}



void init_all(void)
{
    /* Initialize sysclk to FMPLL0@64 MHz */
    Mcu_Sysclk_Init();

    /* Initialise LinFlex 0 to 115200bd  */
    SIUL2.MSCR[18].B.SSS = 1;       /* Pin assigned to LINFlex0 Tx */
    SIUL2.MSCR[18].B.OBE = 1;       /* Output buffer enable */
    SIUL2.MSCR[18].B.SRC = 3;       /* Pad PB2: Maximum slew rate */
    SIUL2.IMCR[165].B.SSS = 1;      /* Pin assigned to LINFlex0 Rx */
    SIUL2.MSCR[19].B.IBE = 1;       /* Input buffer enable */

    Mcu_Linflex_0_Init();           /* Initialize LinFlex 0 */

    /* Initialise FlexCAN 0 to 500kbd  */
    //SIUL2.MSCR[16].B.SSS = 1;       /* Pin assigned to FlexCAN0 Tx */
    //SIUL2.MSCR[16].B.OBE = 1;       /* Output buffer enable */
    //SIUL2.MSCR[16].B.SRC = 3;       /* Pad PB0: Maximum slew rate */
    //SIUL2.IMCR[32].B.SSS = 2;       /* Pin assigned to FlexCAN0 Rx */
    //SIUL2.MSCR[17].B.IBE = 1;       /* Input buffer enable */
    //Mcu_FlexCAN_0_Init();           /* Initialize FlexCAN 0 */

    /* Configure and Enable Interrupts */
    xcptn_xmpl ();

    /* Ensure INTC's current priority is 0 */
    INTC_0.CPR0.B.PRI = 0;

    /* Disable data cache to prevent mismatch between FreeMASTER host part and embedded part memory access */
    __asm__(" mtspr 1010, 2 "); /* l1csr0 = 2 */
}

int main(void)
{
	init_all();

	FMSTR_Init();

	SIUL2.MSCR[45].B.OBE =1;

	/* Loop forever */
	for(;;) {

	   	counter++;
	   	if (counter == MAXCNT)
	   	{
	   		SIUL2.GPDO[45].R ^= 1;
	   		counter = 0;
	   	}
	   	FMSTR_Poll();
	}
}
